"use strict";
class Point {
    constructor() {
        this.lat = 0.00;
        this.lng = 0.00;
    }
}
var OverlayType;
(function (OverlayType) {
    OverlayType[OverlayType["POINT"] = 0] = "POINT";
    OverlayType[OverlayType["CIRCLE"] = 1] = "CIRCLE";
    OverlayType[OverlayType["POLYGON"] = 2] = "POLYGON";
})(OverlayType || (OverlayType = {}));
class OverlayData {
    constructor(type, name, desc, points, options) {
        this.type = type;
        this.name = name;
        this.desc = desc;
        this.points = points;
        this.options = options;
    }
}
class OverlayBase {
    constructor(name, desc, points, options) {
        this.name = name;
        this.desc = desc;
        this.points = points;
        this.options = options;
    }
    add(map) {
        this.self.addTo(map);
    }
    remove(map) {
        this.self.removeFrom(map);
    }
}
class Marker extends OverlayBase {
    constructor(name, desc, point, options) {
        super(name, desc, [point], options);
        this.self = L.marker(point);
        this.self.bindPopup(`<h3>${name}</h3><p>${desc}</p>`);
    }
}
class Circle extends OverlayBase {
    constructor(name, desc, point, options) {
        super(name, desc, [point], options);
        this.self = L.circle(point, options);
        this.self.bindPopup(`<h3>${name}</h3><p>${desc}</p>`);
    }
}
class Polygon extends OverlayBase {
    constructor(name, desc, points, options) {
        super(name, desc, points, options);
        this.self = L.polygon(points, options);
        this.self.bindPopup(`<h3>${name}</h3><p>${desc}</p>`);
    }
}
class Polyline extends OverlayBase {
    constructor() {
        super("", "", [], {});
        this.self = L.polyline([]);
    }
    insertPoint(pt) {
        this.self.addLatLng(pt);
        this.points.push(pt);
    }
    clearPoints() {
        this.points = [];
        this.self.setLatLngs([]);
    }
    numPoints() {
        return this.self.getLatLngs().length;
    }
}
class OverlayState {
    constructor() {
        this.markers = [];
        this.circles = [];
        this.polygons = [];
        this.polyline = new Polyline();
    }
    static load() {
        const store = localStorage.getItem("overlay_state");
        if (store) {
            const model = JSON.parse(store);
            return {
                markers: model.markers.map((m) => OverlayState.fromData(m)),
                circles: model.circles.map((c) => OverlayState.fromData(c)),
                polygons: model.polygons.map((p) => OverlayState.fromData(p)),
                polyline: new Polyline(),
            };
        }
        else {
            return new OverlayState();
        }
    }
    static save(overlayState) {
        localStorage.setItem("overlay_state", JSON.stringify({
            markers: overlayState.markers.map((m) => OverlayState.toData(m)),
            circles: overlayState.circles.map((c) => OverlayState.toData(c)),
            polygons: overlayState.polygons.map((p) => OverlayState.toData(p)),
        }));
    }
    static clear(overlayState, map) {
        overlayState.markers.forEach((m) => m.remove(map));
        overlayState.circles.forEach((c) => c.remove(map));
        overlayState.polygons.forEach((p) => p.remove(map));
        return new OverlayState();
    }
    static toData(source) {
        let type = OverlayType.POINT;
        if (source.points.length > 1) {
            type = OverlayType.POLYGON;
        }
        else if (source.options.radius) {
            type = OverlayType.CIRCLE;
        }
        return new OverlayData(type, source.name, source.desc, source.points, source.options);
    }
    static fromData(data) {
        switch (data.type) {
            case OverlayType.POINT:
                return new Marker(data.name, data.desc, data.points[0], data.options);
            case OverlayType.CIRCLE:
                return new Circle(data.name, data.desc, data.points[0], data.options);
            case OverlayType.POLYGON:
                return new Polygon(data.name, data.desc, data.points, data.options);
        }
    }
}
class TileLayerWrapper {
    constructor(name, self) {
        this.visible = false;
        this.self = self;
        this.name = name;
    }
    static constructLayer(name, self) {
        const wrapper = new TileLayerWrapper(name, self);
        TileLayerWrapper.layers.push(wrapper);
        return wrapper;
    }
    static getActiveLayer() {
        for (const l of TileLayerWrapper.layers) {
            if (l.visible == true) {
                return l.name;
            }
        }
        return null;
    }
    static enableOnly(self, map) {
        for (const l of TileLayerWrapper.layers) {
            if (l.visible) {
                l.self.removeFrom(map);
                l.visible = false;
            }
            if (l.name == self) {
                l.self.addTo(map);
                l.visible = true;
            }
        }
    }
}
TileLayerWrapper.layers = new Array();
class TextUtils {
    static decodeHTML(text) {
        const textArea = document.createElement('textarea');
        textArea.innerHTML = text;
        return textArea.value;
    }
    static encodeHTML(text) {
        const textArea = document.createElement('textarea');
        textArea.innerText = text;
        return textArea.innerHTML;
    }
}
class CreateOverlayModal {
    constructor() {
        const _this = this;
        const closeBtn = document.getElementById("createOverlay-closeBtn");
        if (closeBtn) {
            closeBtn.onclick = () => { _this.setVisible(false); };
        }
    }
    self() {
        return document.getElementById("createOverlay-container");
    }
    title() {
        return document.getElementById("createOverlay-title");
    }
    content() {
        return document.getElementById("createOverlay-content");
    }
    submitBtn() {
        return document.getElementById("createOverlay-submitBtn");
    }
    radiusContainer() {
        return document.getElementById("radius-container");
    }
    nameField() {
        var _a, _b;
        return (_b = (_a = document.getElementById("createOverlay-name")) === null || _a === void 0 ? void 0 : _a.value) !== null && _b !== void 0 ? _b : "";
    }
    descField() {
        var _a, _b;
        return (_b = (_a = document.getElementById("createOverlay-desc")) === null || _a === void 0 ? void 0 : _a.value) !== null && _b !== void 0 ? _b : "";
    }
    radiusField() {
        var _a, _b;
        return (_b = (_a = document.getElementById("createOverlay-radius")) === null || _a === void 0 ? void 0 : _a.value) !== null && _b !== void 0 ? _b : "";
    }
    visible() {
        var _a;
        return ((_a = this.self()) === null || _a === void 0 ? void 0 : _a.style.display) != "none";
    }
    setVisible(v) {
        const modal = this.self();
        if (modal) {
            modal.style.display = v ? "block" : "none";
        }
    }
    clearInputs() {
        const name = document.getElementById("createOverlay-name");
        const desc = document.getElementById("createOverlay-desc");
        const radius = document.getElementById("createOverlay-radius");
        if (name === null || name === void 0 ? void 0 : name.value) {
            name.value = "";
        }
        if (desc === null || desc === void 0 ? void 0 : desc.value) {
            desc.value = "";
        }
        if (Number(radius === null || radius === void 0 ? void 0 : radius.value) != 500) {
            radius.value = "500";
        }
    }
    setState(state, args) {
        const _this = this;
        const title = this.title();
        const radiusContainer = _this.radiusContainer();
        const submitBtn = _this.submitBtn();
        _this.clearInputs();
        if (radiusContainer) {
            radiusContainer.style.display = state == OverlayType.CIRCLE ? "block" : "none";
        }
        switch (state) {
            case OverlayType.POINT:
                if (title) {
                    title.innerHTML = "Add Marker";
                }
                if (submitBtn) {
                    submitBtn.onclick = () => {
                        const name = TextUtils.encodeHTML(_this.nameField());
                        const desc = TextUtils.encodeHTML(_this.descField());
                        if (name.trim().length < 1 || desc.trim().length < 1) {
                            return;
                        }
                        const point = new Marker(name, desc, args.latlng, { title: name, alt: name });
                        point.add(args.map);
                        args.overlays.markers.push(point);
                        _this.setVisible(false);
                    };
                }
                break;
            case OverlayType.CIRCLE:
                if (title) {
                    title.innerHTML = "Add Circle";
                }
                if (submitBtn) {
                    submitBtn.onclick = () => {
                        const radius = _this.radiusField();
                        const name = TextUtils.encodeHTML(_this.nameField());
                        const desc = TextUtils.encodeHTML(_this.descField());
                        if (name.trim().length < 1 || desc.trim().length < 1) {
                            return;
                        }
                        const circle = new Circle(name, desc, args.latlng, { radius: Number(radius) || 500 });
                        circle.add(args.map);
                        args.overlays.circles.push(circle);
                        _this.setVisible(false);
                    };
                }
                break;
            case OverlayType.POLYGON:
                if (title) {
                    title.innerHTML = "Add Polygon";
                }
                if (submitBtn) {
                    submitBtn.onclick = () => {
                        const name = TextUtils.encodeHTML(_this.nameField());
                        const desc = TextUtils.encodeHTML(_this.descField());
                        if (name.trim().length < 1 || desc.trim().length < 1) {
                            return;
                        }
                        const polygon = new Polygon(name, desc, args.points, {});
                        polygon.add(args.map);
                        args.overlays.polygons.push(polygon);
                        _this.setVisible(false);
                    };
                }
                break;
        }
    }
}
class CancelModal {
    self() {
        return document.getElementById("cancel-container");
    }
    cancelMsg() {
        return document.getElementById("cancel-msg");
    }
    cancelBtn() {
        return document.getElementById("cancel-btn");
    }
    visible() {
        var _a;
        return ((_a = this.self()) === null || _a === void 0 ? void 0 : _a.style.display) != "none";
    }
    setVisible(v) {
        const modal = this.self();
        if (modal) {
            modal.style.display = v ? "block" : "none";
        }
    }
    setMsg(s) {
        const msg = this.cancelMsg();
        if (msg) {
            msg.innerText = s;
        }
    }
}
class OKCancelModal {
    self() {
        return document.getElementById("confirm-container");
    }
    confirmMsg() {
        return document.getElementById("confirm-msg");
    }
    okBtn() {
        return document.getElementById("yes-btn");
    }
    cancelBtn() {
        return document.getElementById("no-btn");
    }
    visible() {
        var _a;
        return ((_a = this.self()) === null || _a === void 0 ? void 0 : _a.style.display) != "none";
    }
    setVisible(v) {
        const modal = this.self();
        if (modal) {
            modal.style.display = v ? "block" : "none";
        }
    }
    setMsg(s) {
        const msg = this.confirmMsg();
        if (msg) {
            msg.innerText = s;
        }
    }
}
class ModalCollection {
    constructor(createOverlay, cancel, okCancel) {
        this.createOverlay = createOverlay;
        this.cancel = cancel;
        this.okCancel = okCancel;
    }
    closeAll() {
        this.createOverlay.setVisible(false);
        this.cancel.setVisible(false);
        this.okCancel.setVisible(false);
    }
}
class MapHandler {
    constructor(map, overlays, layers, modals) {
        this.map = map;
        this.overlays = overlays;
        this.layers = layers;
        this.modals = modals;
    }
    static init(map, overlays, layers, modals) {
        if (!MapHandler.instance) {
            MapHandler.instance = new MapHandler(map, overlays, layers, modals);
        }
    }
    static setButtonClick(btnId, handler) {
        const button = document.getElementById(btnId);
        if (button) {
            button.onclick = handler;
        }
    }
    static resetMapClick() {
        const self = MapHandler.instance;
        if (self) {
            try {
                const addPointBtn = document.getElementById("addPoint-btn");
                if (addPointBtn) {
                    addPointBtn.classList.remove("activeBtn");
                }
                self.map.off("click", MapHandler.addMarker);
            }
            catch (_a) { }
            try {
                const addCircleBtn = document.getElementById("addCircle-btn");
                if (addCircleBtn) {
                    addCircleBtn.classList.remove("activeBtn");
                }
                self.map.off("click", MapHandler.addCircle);
            }
            catch (_b) { }
            try {
                const addPolygonBtn = document.getElementById("addPolygon-btn");
                if (addPolygonBtn) {
                    addPolygonBtn.classList.remove("activeBtn");
                }
                self.map.off("click", MapHandler.polygonAddPoint);
            }
            catch (_c) { }
        }
    }
    static addMarker(e) {
        const self = MapHandler.instance;
        if (self) {
            self.modals.cancel.setVisible(false);
            self.modals.createOverlay.setState(OverlayType.POINT, {
                latlng: e.latlng,
                map: self.map,
                overlays: self.overlays,
            });
            MapHandler.resetMapClick();
            self.modals.createOverlay.setVisible(true);
        }
    }
    static addCircle(e) {
        const self = MapHandler.instance;
        if (self) {
            self.modals.cancel.setVisible(false);
            self.modals.createOverlay.setState(OverlayType.CIRCLE, {
                latlng: e.latlng,
                map: self.map,
                overlays: self.overlays,
            });
            MapHandler.resetMapClick();
            self.modals.createOverlay.setVisible(true);
        }
    }
    static addPolygon(e) {
        const self = MapHandler.instance;
        if (self) {
            self.modals.okCancel.setVisible(false);
            self.modals.createOverlay.setState(OverlayType.POLYGON, {
                points: self.overlays.polyline.points,
                map: self.map,
                overlays: self.overlays,
            });
            MapHandler.resetMapClick();
            self.overlays.polyline.clearPoints();
            self.modals.createOverlay.setVisible(true);
        }
    }
    static circleCollect(e) {
        const self = MapHandler.instance;
        if (self) {
            self.modals.closeAll();
            MapHandler.resetMapClick();
            e.target.classList.add("activeBtn");
            self.map.on("click", MapHandler.addCircle);
            const cancelBtn = self.modals.cancel.cancelBtn();
            if (cancelBtn) {
                cancelBtn.onclick = () => {
                    self.modals.closeAll();
                    MapHandler.resetMapClick();
                };
            }
            self.modals.cancel.setMsg("Placing circle");
            self.modals.cancel.setVisible(true);
        }
    }
    static markerCollect(e) {
        const self = MapHandler.instance;
        if (self) {
            self.modals.closeAll();
            MapHandler.resetMapClick();
            e.target.classList.add("activeBtn");
            self.map.on("click", MapHandler.addMarker);
            const cancelBtn = self.modals.cancel.cancelBtn();
            if (cancelBtn) {
                cancelBtn.onclick = () => {
                    self.modals.closeAll();
                    MapHandler.resetMapClick();
                };
            }
            self.modals.cancel.setMsg("Placing marker");
            self.modals.cancel.setVisible(true);
        }
    }
    static polygonCollect(e) {
        const self = MapHandler.instance;
        if (self) {
            self.modals.closeAll();
            MapHandler.resetMapClick();
            e.target.classList.add("activeBtn");
            // show cancel -- on cancel, clear polyline and reset map handling
            const cancelBtn = self.modals.cancel.cancelBtn();
            if (cancelBtn) {
                cancelBtn.onclick = MapHandler.polygonClearPoints;
            }
            self.modals.cancel.setMsg("Creating polygon");
            self.modals.cancel.setVisible(true);
            self.map.on("click", MapHandler.polygonAddPoint);
        }
    }
    static polygonClearPoints(e) {
        const self = MapHandler.instance;
        if (self) {
            self.modals.closeAll();
            self.overlays.polyline.clearPoints();
            MapHandler.resetMapClick();
        }
    }
    static polygonAddPoint(e) {
        const self = MapHandler.instance;
        if (self) {
            self.overlays.polyline.insertPoint(e.latlng);
            if (self.overlays.polyline.numPoints() >= 3) {
                self.modals.cancel.setVisible(false);
                const okBtn = self.modals.okCancel.okBtn();
                if (okBtn) {
                    okBtn.onclick = MapHandler.addPolygon;
                }
                const cancelBtn = self.modals.okCancel.cancelBtn();
                if (cancelBtn) {
                    cancelBtn.onclick = MapHandler.polygonClearPoints;
                }
                self.modals.okCancel.setMsg("Creating polygon");
                self.modals.okCancel.setVisible(true);
            }
        }
    }
    static overlaySave(e) {
        const self = MapHandler.instance;
        if (self) {
            self.modals.closeAll();
            MapHandler.resetMapClick();
            e.target.classList.add("activeBtn");
            self.modals.okCancel.setMsg("Save current map overlays?");
            const okBtn = self.modals.okCancel.okBtn();
            if (okBtn) {
                okBtn.onclick = () => {
                    OverlayState.save(self.overlays);
                    self.modals.okCancel.setVisible(false);
                    // show info modal "Save complete" 
                };
            }
            const cancelBtn = self.modals.okCancel.cancelBtn();
            if (cancelBtn) {
                cancelBtn.onclick = () => {
                    self.modals.okCancel.setVisible(false);
                };
            }
            self.modals.okCancel.setVisible(true);
        }
    }
    static overlayClear(e) {
        const self = MapHandler.instance;
        if (self) {
            self.modals.closeAll();
            MapHandler.resetMapClick();
            e.target.classList.add("activeBtn");
            self.modals.okCancel.setMsg("Clear all map overlays (will not affect saved data)?");
            const okBtn = self.modals.okCancel.okBtn();
            if (okBtn) {
                okBtn.onclick = () => {
                    self.overlays = OverlayState.clear(self.overlays, self.map);
                    self.modals.okCancel.setVisible(false);
                };
            }
            const cancelBtn = self.modals.okCancel.cancelBtn();
            if (cancelBtn) {
                cancelBtn.onclick = () => {
                    self.modals.okCancel.setVisible(false);
                };
            }
            self.modals.okCancel.setVisible(true);
        }
    }
    static swapTiles(e) {
        const self = MapHandler.instance;
        if (self) {
            if (TileLayerWrapper.getActiveLayer() == "satelliteLayer") {
                TileLayerWrapper.enableOnly("streetLayer", self.map);
            }
            else {
                TileLayerWrapper.enableOnly("satelliteLayer", self.map);
            }
        }
    }
}
MapHandler.instance = null;
function init() {
    var _a;
    let overlays = (_a = OverlayState.load()) !== null && _a !== void 0 ? _a : new OverlayState();
    const map = L.map('map').setView([35.6653, -105.9507], 13);
    const streetLayer = TileLayerWrapper.constructLayer("streetLayer", L.tileLayer('https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', {
        maxZoom: 19,
        attribution: "street map tiles &copy; OpenStreetMap"
    }));
    const satelliteLayer = TileLayerWrapper.constructLayer("satelliteLayer", L.tileLayer('https://server.arcgisonline.com/ArcGIS/rest/services/World_Imagery/MapServer/tile/{z}/{y}/{x}', {
        maxZoom: 19,
        attribution: "satellite tiles &copy; Esri"
    }));
    TileLayerWrapper.enableOnly("streetLayer", map);
    overlays.markers.forEach(m => m.add(map));
    overlays.circles.forEach(m => m.add(map));
    overlays.polygons.forEach(m => m.add(map));
    overlays.polyline.add(map);
    const modals = new ModalCollection(new CreateOverlayModal(), new CancelModal(), new OKCancelModal());
    MapHandler.init(map, overlays, TileLayerWrapper.layers, modals);
    MapHandler.setButtonClick("addPoint-btn", MapHandler.markerCollect);
    MapHandler.setButtonClick("addCircle-btn", MapHandler.circleCollect);
    MapHandler.setButtonClick("addPolygon-btn", MapHandler.polygonCollect);
    MapHandler.setButtonClick("save-btn", MapHandler.overlaySave);
    MapHandler.setButtonClick("clear-btn", MapHandler.overlayClear);
    MapHandler.setButtonClick("tiles-btn", MapHandler.swapTiles);
}
init();
